<?php

class Maarch_Sniffs_NamingConventions_ValidClassNameSniff implements PHP_CodeSniffer_Sniff
{


    /**
     * Returns an array of tokens this test wants to listen for.
     *
     * @return array
     */
    public function register()
    {
        return array(
                T_CLASS,
                T_INTERFACE,
               );

    }


    /**
     * Processes this test, when one of its tokens is encountered.
     *
     * @param PHP_CodeSniffer_File $phpcsFile The current file being processed.
     * @param int                  $stackPtr  The position of the current token in the
     *                                        stack passed in $tokens.
     *
     * @return void
     */
    public function process(PHP_CodeSniffer_File $phpcsFile, $stackPtr)
    {
        $tokens = $phpcsFile->getTokens();
        
        if (isset($tokens[$stackPtr]['scope_opener']) === false) {
            $error = 'Possible parse error: %s missing opening or closing brace';
            $data  = array($tokens[$stackPtr]['content']);
            $phpcsFile->addWarning($error, $stackPtr, 'MissingBrace', $data);
            return;
        }

        // Determine the name of the class or interface. Note that we cannot
        // simply look for the first T_STRING because a class name
        // starting with the number will be multiple tokens.
        $opener    = $tokens[$stackPtr]['scope_opener'];
        $nameStart = $phpcsFile->findNext(T_WHITESPACE, ($stackPtr + 1), $opener, true);
        $nameEnd   = $phpcsFile->findNext(T_WHITESPACE, $nameStart, $opener);
        $name      = trim($phpcsFile->getTokensAsString($nameStart, ($nameEnd - $nameStart)));

        // Check for camel caps format.
        //$valid = PHP_CodeSniffer::isCamelCaps($name, true, true, false);
        $valid = true;
        if (!preg_match("/^(([A-Z]([a-z]+))+_?)+[a-z]$/", $name)) {
            $valid = false;
        }
        if ($valid === false) {
            $type  = ucfirst($tokens[$stackPtr]['content']);
            $error = '%s name "%s" is not in camel caps format';
            $data  = array(
                      $type,
                      $name,
                     );
            $phpcsFile->addError($error, $stackPtr, 'NotCamelCaps', $data);
        }
        if ($tokens[$stackPtr]['code'] == T_INTERFACE) {
            if (!preg_match("/[a-z]Interface$/", $name)){
                $data = array($name);
                $error = 'Interface name "%s" must end with "Interface"';
                $phpcsFile->addError($error, $stackPtr, 'BadInterfaceName', $data);
            }
        }
        
        $abstractPos = $phpcsFile->findPrevious(
            T_ABSTRACT, 
            max($stackPtr - 2, 0),
            max($stackPtr - 5, 0)
        );
        if ($abstractPos !== false) {
            if (!preg_match("/[a-z]Abstract$/", $name)){
                $data = array($name);
                $error = 'Abstract class name "%s" must end with "Abstract"';
                $phpcsFile->addError($error, $stackPtr, 'BadAbstractClassName', $data);
            }
        }
    }//end process()


}//end class